"""Adapted from
https://earth.bsc.es/gitlab/digital-twins/de_340-2/workflow/-/blob/main/lib/runscript/run_opa.py
"""

from argparse import ArgumentParser
import cProfile
import os
import sys

from pprint import pprint

from one_pass.opa import Opa
from opa_tools import get_request, benchmark_setup
from gsv import GSVRetriever

def _get_parser():
    parser = ArgumentParser(description="Runscript for OPA job.")
    parser.add_argument(
        "-r", "--request",
        type=int,
        required=True,
        help="Input request number"
    )
    parser.add_argument(
        "--read_from_databridge",
        type=bool,
        help="Read from databridge",
        default=False
    )
    parser.add_argument(
        "-b", "--benchmark",
        action="store_true",
        help="Switch on benchmarking",
    )
    return parser

def run_bias_adjustment(oparequest):  # type: ignore
    raise NotImplementedError(
        "OPA with bias adjustment not implemented in OPA-Tools package. "
        "Refer to the original script for implementation."
    )

def main(argv=None):
    if argv is None:
        argv = sys.argv

    args = _get_parser().parse_args(argv[1:])
    request = args.request
    read_from_databridge = args.read_from_databridge
    profile = (
        cProfile.Profile()
        if args.benchmark
        else False
    )

    gsvrequest, oparequest = get_request(request)
    print("\nGSV REQUEST:\n============\n")
    pprint(gsvrequest)
    print("\nOPA REQUEST:\n============\n")
    pprint(oparequest)

    if profile:
        benchmark_path = benchmark_setup(environ=True, gsvrequest=gsvrequest, oparequest=oparequest)

    gsv = GSVRetriever()
    data = gsv.request_data(
        gsvrequest,
        use_stream_iterator=read_from_databridge
    )
    if data is None:
        raise SystemExit(
            "No data was retreived with passed request pair."
        )

    if oparequest["stat"] == "raw" and oparequest["bias_adjustment"] is True:
        run_bias_adjustment(oparequest)

    else:
        if profile:
            profile.enable()
        opa_stat = Opa(oparequest)
        opa_stat.compute(data)
        if profile:
            profile.disable()
            profile.dump_stats(benchmark_path / "opa.prof")

if __name__ == "__main__":
    if os.environ.get("FDB_HOME") is None:
        raise EnvironmentError(
            "Please, set variable 'FDB_HOME'."
        )
    main()
